@@ -2,9 +2,8 @@
 import os
 import sys
 
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from fnmatch import fnmatch
-from optparse import make_option
 from importlib import import_module
 
 import django
@@ -13,14 +12,12 @@ import django.template
 from django.template import Context
 from django.utils import six
 from django.template.loader import get_template  # noqa Leave this in to preload template locations
-from django.template.utils import InvalidTemplateEngineError
 from django.template import engines
 
 from compressor.cache import get_offline_hexdigest, write_offline_manifest
 from compressor.conf import settings
 from compressor.exceptions import (OfflineGenerationError, TemplateSyntaxError,
                                    TemplateDoesNotExist)
-from compressor.templatetags.compress import CompressorNode
 from compressor.utils import get_mod_func
 
 if six.PY3:
@@ -36,23 +33,23 @@ else:
 
 class Command(BaseCommand):
     help = "Compress content outside of the request/response cycle"
-    option_list = BaseCommand.option_list + (
-        make_option('--extension', '-e', action='append', dest='extensions',
-            help='The file extension(s) to examine (default: ".html", '
-                'separate multiple extensions with commas, or use -e '
-                'multiple times)'),
-        make_option('-f', '--force', default=False, action='store_true',
-            help="Force the generation of compressed content even if the "
-                "COMPRESS_ENABLED setting is not True.", dest='force'),
-        make_option('--follow-links', default=False, action='store_true',
-            help="Follow symlinks when traversing the COMPRESS_ROOT "
-                "(which defaults to STATIC_ROOT). Be aware that using this "
-                "can lead to infinite recursion if a link points to a parent "
-                "directory of itself.", dest='follow_links'),
-        make_option('--engine', default="django", action="store",
-            help="Specifies the templating engine. jinja2 or django",
-            dest="engine"),
-    )
+
+    def add_arguments(self, parser):
+        parser.add_argument('--extension', '-e', action='append', dest='extensions',
+                            help='The file extension(s) to examine (default: ".html", '
+                                 'separate multiple extensions with commas, or use -e '
+                                 'multiple times)')
+        parser.add_argument('-f', '--force', default=False, action='store_true',
+                            help="Force the generation of compressed content even if the "
+                                 "COMPRESS_ENABLED setting is not True.", dest='force')
+        parser.add_argument('--follow-links', default=False, action='store_true',
+                            help="Follow symlinks when traversing the COMPRESS_ROOT "
+                                 "(which defaults to STATIC_ROOT). Be aware that using this "
+                                 "can lead to infinite recursion if a link points to a parent "
+                                 "directory of itself.", dest='follow_links')
+        parser.add_argument('--engine', default="django", action="store",
+                            help="Specifies the templating engine. jinja2 or django",
+                            dest="engine")
 
     def get_loaders(self):
         template_source_loaders = []
@@ -107,10 +104,11 @@ class Command(BaseCommand):
         verbosity = int(options.get("verbosity", 0))
         if not log:
             log = StringIO()
-        if not settings.TEMPLATE_LOADERS:
+        if not self.get_loaders():
             raise OfflineGenerationError("No template loaders defined. You "
                                          "must set TEMPLATE_LOADERS in your "
-                                         "settings.")
+                                         "settings or set 'loaders' in your "
+                                         "TEMPLATES dictionary.")
         templates = set()
         if engine == 'django':
             paths = set()
@@ -156,6 +154,18 @@ class Command(BaseCommand):
         if verbosity > 1:
             log.write("Found templates:\n\t" + "\n\t".join(templates) + "\n")
 
+        contexts = settings.COMPRESS_OFFLINE_CONTEXT
+        if isinstance(contexts, six.string_types):
+            try:
+                module, function = get_mod_func(contexts)
+                contexts = getattr(import_module(module), function)()
+            except (AttributeError, ImportError, TypeError) as e:
+                raise ImportError("Couldn't import offline context function %s: %s" %
+                                  (settings.COMPRESS_OFFLINE_CONTEXT, e))
+        elif not isinstance(contexts, (list, tuple)):
+            contexts = [contexts]
+        contexts = list(contexts) # evaluate generator
+
         parser = self.__get_parser(engine)
         compressor_nodes = OrderedDict()
         for template_name in templates:
@@ -177,16 +187,23 @@ class Command(BaseCommand):
                 if verbosity > 0:
                     log.write("UnicodeDecodeError while trying to read "
                               "template %s\n" % template_name)
-            try:
-                nodes = list(parser.walk_nodes(template))
-            except (TemplateDoesNotExist, TemplateSyntaxError) as e:
-                # Could be an error in some base template
-                if verbosity > 0:
-                    log.write("Error parsing template %s: %s\n" % (template_name, e))
                 continue
-            if nodes:
-                template.template_name = template_name
-                compressor_nodes.setdefault(template, []).extend(nodes)
+
+            for context_dict in contexts:
+                context = parser.get_init_context(context_dict)
+                context = Context(context)
+                try:
+                    nodes = list(parser.walk_nodes(template, context=context))
+                except (TemplateDoesNotExist, TemplateSyntaxError) as e:
+                    # Could be an error in some base template
+                    if verbosity > 0:
+                        log.write("Error parsing template %s: %s\n" % (template_name, e))
+                    continue
+                if nodes:
+                    template.template_name = template_name
+                    template_nodes = compressor_nodes.setdefault(template, OrderedDict())
+                    for node in nodes:
+                        template_nodes.setdefault(node, []).append(context)
 
         if not compressor_nodes:
             raise OfflineGenerationError(
@@ -199,36 +216,23 @@ class Command(BaseCommand):
                       "\n\t".join((t.template_name
                                    for t in compressor_nodes.keys())) + "\n")
 
-        contexts = settings.COMPRESS_OFFLINE_CONTEXT
-        if isinstance(contexts, six.string_types):
-            try:
-                module, function = get_mod_func(contexts)
-                contexts = getattr(import_module(module), function)()
-            except (AttributeError, ImportError, TypeError) as e:
-                raise ImportError("Couldn't import offline context function %s: %s" %
-                                  (settings.COMPRESS_OFFLINE_CONTEXT, e))
-        elif not isinstance(contexts, (list, tuple)):
-            contexts = [contexts]
-
         log.write("Compressing... ")
-        block_count = context_count = 0
+        block_count = 0
+        compressed_contexts = []
         results = []
         offline_manifest = OrderedDict()
+        for template, nodes in compressor_nodes.items():
+            template._log = log
+            template._log_verbosity = verbosity
 
-        for context_dict in contexts:
-            context_count += 1
-            init_context = parser.get_init_context(context_dict)
-
-            for template, nodes in compressor_nodes.items():
-                context = Context(init_context)
-                template._log = log
-                template._log_verbosity = verbosity
-
-                if not parser.process_template(template, context):
-                    continue
-
-                for node in nodes:
+            for node, contexts in nodes.items():
+                for context in contexts:
+                    if context not in compressed_contexts:
+                        compressed_contexts.append(context)
                     context.push()
+                    if not parser.process_template(template, context):
+                        continue
+
                     parser.process_node(template, context, node)
                     rendered = parser.render_nodelist(template, context, node)
                     key = get_offline_hexdigest(rendered)
@@ -248,6 +252,7 @@ class Command(BaseCommand):
 
         write_offline_manifest(offline_manifest)
 
+        context_count = len(compressed_contexts)
         log.write("done\nCompressed %d block(s) from %d template(s) for %d context(s).\n" %
                   (block_count, len(compressor_nodes), context_count))
         return block_count, results
